from contextlib import contextmanager

from sqlalchemy.orm import sessionmaker


@contextmanager
def session_scope(session_class: sessionmaker) -> None:
    session = session_class()
    try:
        yield session
        session.commit()
    except:
        session.rollback()
        raise



from chpass.dal.db_connection import DBConnection
from chpass.dal.models.history import History


class HistoryTableAdapter(object):
    def __init__(self, db_connection: DBConnection) -> None:
        self._db_connection = db_connection

    def get_chrome_history(self, serializable: bool = False) -> list:
        return self._db_connection.select(History, serializable=serializable)



from typing import List, Callable


def convert_bytes_in_data(data: List[dict], convert_callback: Callable, byte_columns: list = None) -> List[dict]:
    if byte_columns is None:
        byte_columns = []
    for row in data:
        for byte_column in byte_columns:
            row[byte_column] = convert_callback(row[byte_column])
    return data


def str_bytes_to_bytes(str_bytes: str) -> bytes:
    list_str_bytes = str_bytes[1:-1].split(",")
    list_bytes = [int(str_bytes) for str_bytes in list_str_bytes]
    return bytes(list_bytes)



import sys

from chpass.cli import parse_args
from chpass.config import OUTPUT_FILE_PATHS, DB_PROTOCOL
from chpass.core.object_factory import ObjectFactory
from chpass.dal.chrome_db_adapter import ChromeDBAdapter
from chpass.dal.db_connection import DBConnection
from chpass.dal.db_adapters.history_db_adapter import HistoryDBAdapter
from chpass.dal.db_adapters.logins_db_adapter import LoginsDBAdapter
from chpass.dal.db_adapters.top_sites_db_adapter import TopSitesDBAdapter
from chpass.exceptions.file_adapter_not_supported_exception import FileAdapterNotSupportedException
from chpass.services.chrome import export_chrome_data, import_chrome_passwords
from chpass.services.file_adapters.csv_file_adapter import CsvFileAdapter
from chpass.services.file_adapters.json_file_adapter import JsonFileAdapter
from chpass.core.interfaces import file_adapter_interface
from chpass.services.path import get_chrome_logins_path, get_chrome_history_path, get_chrome_top_sites_path


def create_file_adapter(file_adapter_type: str) -> file_adapter_interface:
    object_factory = ObjectFactory()
    object_factory.register_builder("json", JsonFileAdapter)
    object_factory.register_builder("csv", CsvFileAdapter)
    return object_factory.create(file_adapter_type, exception=FileAdapterNotSupportedException)


def create_chrome_db_adapter(protocol: str, os_user: str) -> ChromeDBAdapter:
    logins_db_connection = DBConnection()
    history_db_connection = DBConnection()
    top_sites_db_connection = DBConnection()
    logins_db_connection.connect(protocol, get_chrome_logins_path(os_user))
    history_db_connection.connect(protocol, get_chrome_history_path(os_user))
    top_sites_db_connection.connect(protocol, get_chrome_top_sites_path(os_user))
    logins_db_adapter = LoginsDBAdapter(logins_db_connection)
    history_db_adapter = HistoryDBAdapter(history_db_connection)
    top_sites_db_adapter = TopSitesDBAdapter(top_sites_db_connection)
    return ChromeDBAdapter(logins_db_adapter, history_db_adapter, top_sites_db_adapter)


def start(args=None) -> None:
    if args:
        args = parse_args(args)
    else:
        args = parse_args(sys.argv[1:])
    file_adapter = create_file_adapter(args.file_adapter)
    output_file_paths = OUTPUT_FILE_PATHS[args.file_adapter]
    chrome_db_adapter = create_chrome_db_adapter(DB_PROTOCOL, args.user)
    mode_actions = {
        "export": lambda: export_chrome_data(chrome_db_adapter, args.destination_folder, file_adapter,
                                             output_file_paths, args.user, args.export_kind),
        "import": lambda: import_chrome_passwords(chrome_db_adapter, args.from_file, file_adapter)
    }
    mode_actions[args.mode]()
    chrome_db_adapter.close()


if __name__ == "__main__":
    start()



import getpass
import os
import sys

from chpass.config import (
    HISTORY_DB_FILE_PATH,
    LOGINS_DB_FILE_PATH,
    TOP_SITES_DB_FILE_PATH,
    GOOGLE_PICTURE_FILE_PATH,
    CHROME_FOLDER_OS_PATHS
)
from chpass.exceptions.chrome_not_installed_exception import ChromeNotInstalledException
from chpass.exceptions.operating_system_not_supported import OperatingSystemNotSupported
from chpass.exceptions.user_not_found_exception import UserNotFoundException


def get_home_directory(user: str = getpass.getuser()) -> str:
    """Get home directory path of the given user
    :param user: OS user
    :return: Home directory
    :rtype: str
    """
    home_directory = os.path.expanduser("~" + user)
    if not os.path.exists(home_directory):
        raise UserNotFoundException(user)
    return home_directory


def get_chrome_user_folder(user: str = getpass.getuser(), platform=sys.platform) -> str:
    """Get chrome folder path of the given user
    :param user: Chrome user
    :param platform: Operating system
    :return: Chrome user folder
    :rtype: str
    """
    home_directory = get_home_directory(user)
    if platform not in CHROME_FOLDER_OS_PATHS.keys():
        raise OperatingSystemNotSupported(platform)
    chrome_folder_path = CHROME_FOLDER_OS_PATHS[platform]
    chrome_user_folder = os.path.join(home_directory, chrome_folder_path)
    if not os.path.exists(chrome_user_folder):
        raise ChromeNotInstalledException(user)
    return chrome_user_folder


def get_chrome_history_path(user: str = getpass.getuser()) -> str:
    return "/" + os.path.join(get_chrome_user_folder(user), HISTORY_DB_FILE_PATH)


def get_chrome_logins_path(user: str = getpass.getuser()) -> str:
    return "/" + os.path.join(get_chrome_user_folder(user), LOGINS_DB_FILE_PATH)


def get_chrome_top_sites_path(user: str = getpass.getuser()) -> str:
    return "/" + os.path.join(get_chrome_user_folder(user), TOP_SITES_DB_FILE_PATH)


def get_chrome_profile_picture_path(user: str = getpass.getuser()) -> str:
    return os.path.join(get_chrome_user_folder(user), GOOGLE_PICTURE_FILE_PATH)



from dataclasses import dataclass

from sqlalchemy import Integer, String, Column, LargeBinary

from chpass.dal.models.base import Base


@dataclass
class Download(Base):
    __tablename__ = 'downloads'
    id = Column(Integer, primary_key=True)
    guid = Column(String)
    current_path = Column(String)
    target_path = Column(String)
    start_time = Column(Integer)
    received_bytes = Column(Integer)
    total_bytes = Column(Integer)
    state = Column(Integer)
    danger_type = Column(Integer)
    interrupt_reason = Column(Integer)
    hash = Column(LargeBinary)
    end_time = Column(Integer)
    opened = Column(Integer)
    last_access_time = Column(Integer)
    transient = Column(Integer)
    referrer = Column(String)
    site_url = Column(String)
    tab_url = Column(String)
    tab_referrer_url = Column(String)
    http_method = Column(String)
    by_ext_id = Column(String)
    by_ext_name = Column(String)
    etag = Column(String)
    last_modified = Column(String)
    mime_type = Column(String)
    original_mime_type = Column(String)



from chpass.dal.models.history import History


def test_get_history(chrome_db_adapter):
    history = chrome_db_adapter.history_db.history_table.get_chrome_history()
    assert all(isinstance(history_row, History) for history_row in history)


def test_get_history_serializable(chrome_db_adapter):
    history = chrome_db_adapter.history_db.history_table.get_chrome_history(serializable=True)
    assert all(isinstance(history_row, dict) for history_row in history)



from chpass.dal.db_connection import DBConnection
from chpass.dal.models.download import Download


class DownloadsTableAdapter(object):
    def __init__(self, db_connection: DBConnection) -> None:
        self._db_connection = db_connection

    def get_chrome_downloads(self, serializable: bool = False) -> list:
        return self._db_connection.select(Download, serializable=serializable)



from dataclasses import dataclass

from sqlalchemy import Integer, String, Column, Boolean

from chpass.dal.models.base import Base


@dataclass
class History(Base):
    __tablename__ = 'urls'
    id = Column(Integer, primary_key=True)
    url = Column(String)
    title = Column(String)
    visit_count = Column(Integer)
    typed_count = Column(Boolean)
    last_visit_time = Column(Integer)
    hidden = Column(Boolean)



import os
from typing import List

import pytest

from chpass.services.file_adapters.csv_file_adapter import CsvFileAdapter
from chpass.services.file_adapters.json_file_adapter import JsonFileAdapter


@pytest.fixture(scope="module")
def correct_data_list() -> List[dict]:
    return [{
        "column1": "value1",
        "column2": "value2",
        "column3": "value3"
    }]


@pytest.fixture(scope="module")
def invalid_data_list() -> str:
    return "invalid data list"


@pytest.fixture(scope="module")
def not_exist_output_file_path() -> str:
    return "not_exist/test"


@pytest.fixture(scope="module")
def invalid_output_file_path() -> int:
    return -1


@pytest.mark.parametrize("file_adapter, output_file_path", [(CsvFileAdapter(), "test.csv"), (JsonFileAdapter(), "test.json")])
def test_csv_write(file_adapter, output_file_path, correct_data_list):
    file_adapter.write(correct_data_list, output_file_path)
    assert os.path.exists(output_file_path)


@pytest.mark.parametrize("file_adapter, output_file_path", [(CsvFileAdapter(), "test.csv"), (JsonFileAdapter(), "test.json")])
def test_csv_read(file_adapter, output_file_path, correct_data_list):
    result_correct_data_list = file_adapter.read(output_file_path)
    assert result_correct_data_list == correct_data_list
    os.remove(output_file_path)


@pytest.mark.parametrize("file_adapter", [CsvFileAdapter(), JsonFileAdapter()])
def test_csv_write_output_file_path_not_exist(file_adapter, not_exist_output_file_path, correct_data_list):
    with pytest.raises(FileNotFoundError):
        file_adapter.write(correct_data_list, not_exist_output_file_path)
    assert not os.path.exists(not_exist_output_file_path)


@pytest.mark.parametrize("file_adapter", [CsvFileAdapter(), JsonFileAdapter()])
def test_csv_write_invalid_output_file_path(file_adapter, invalid_output_file_path, correct_data_list):
    with pytest.raises(ValueError):
        file_adapter.write(correct_data_list, invalid_output_file_path)
    assert not os.path.exists(invalid_output_file_path)


@pytest.mark.parametrize("file_adapter, output_file_path", [(CsvFileAdapter(), "test.csv"), (JsonFileAdapter(), "test.json")])
def test_csv_write_invalid_data_list(file_adapter, output_file_path, invalid_data_list):
    with pytest.raises(ValueError):
        file_adapter.write(invalid_data_list, output_file_path)
    assert not os.path.exists(output_file_path)



import pytest

from chpass.cli import parse_args
from chpass.config import DEFAULT_EXPORT_DESTINATION_FOLDER, DEFAULT_FILE_ADAPTER


@pytest.fixture(scope="module")
def export_mode() -> str:
    return "export"


@pytest.fixture(scope="module")
def import_mode() -> str:
    return "import"


@pytest.fixture(scope="module")
def from_file() -> str:
    return "passwords.csv"


def test_default_export(export_mode, connected_user):
    args = parse_args([export_mode])
    assert args.mode == export_mode
    assert args.user == connected_user
    assert args.destination_folder == DEFAULT_EXPORT_DESTINATION_FOLDER
    assert args.file_adapter == DEFAULT_FILE_ADAPTER
    assert not hasattr(args, "from_file")


def test_default_import(import_mode, connected_user, from_file):
    args = parse_args([import_mode, "-f", from_file])
    assert args.mode == import_mode
    assert args.user == connected_user
    assert not hasattr(args, "destination_folder")
    assert args.file_adapter == "csv"
    assert args.from_file == from_file


def test_user_flag_export(export_mode, connected_user):
    user = connected_user
    args = parse_args(["-u", user, export_mode])
    assert args.mode == export_mode
    assert args.user == user


def test_user_flag_import(import_mode, from_file, connected_user):
    user = connected_user
    args = parse_args(["-u", user, import_mode, "-f", from_file])
    assert args.mode == import_mode
    assert args.user == user


def test_file_adapter_flag_export(export_mode):
    file_adapter = export_mode
    args = parse_args(["-i", file_adapter, export_mode])
    assert args.mode == export_mode
    assert args.file_adapter == file_adapter


@pytest.fixture
def correct_file_adapter() -> str:
    return "json"


def test_file_adapter_flag_import(import_mode, correct_file_adapter, from_file):
    args = parse_args(["-i", correct_file_adapter, import_mode, "-f", from_file])
    assert args.mode == import_mode
    assert correct_file_adapter == correct_file_adapter


@pytest.fixture
def correct_destination_folder() -> str:
    return "test"


def test_export_destination_folder_flag(export_mode, correct_destination_folder):
    args = parse_args([export_mode, "-d", correct_destination_folder])
    assert args.mode == export_mode
    assert args.destination_folder == correct_destination_folder



from typing import List

import pandas as pd

from chpass.config import CSV_ADAPTER_ENCODING
from chpass.core.interfaces.file_adapter_interface import IFileAdapter
from chpass.services.bytes import str_bytes_to_bytes, convert_bytes_in_data


class CsvFileAdapter(IFileAdapter):
    def write(self, data: List[dict], output_file_path: str, byte_columns: list = None) -> None:
        convert_bytes_in_data(data, list, byte_columns)
        df = pd.DataFrame(data)
        df.to_csv(output_file_path, index=False, encoding=CSV_ADAPTER_ENCODING)

    def read(self, path: str, byte_columns: list = None) -> list:
        df = pd.read_csv(path, encoding=CSV_ADAPTER_ENCODING)
        data = [dict(row[1]) for row in df.iterrows()]
        convert_bytes_in_data(data, str_bytes_to_bytes, byte_columns)
        return data





class ChromeNotInstalledException(Exception):
    def __init__(self, user: str) -> None:
        self._user = user
        message = f"chrome is not installed for the user - {user}"
        super().__init__(message)

    @property
    def user(self) -> str:
        return self._user

    @user.setter
    def user(self, new_user: str) -> None:
        self._user = new_user



import pytest


@pytest.fixture(scope="module")
def import_mode() -> str:
    return "import"


@pytest.fixture(scope="module")
def from_file() -> str:
    return "dist/passwords.csv"


@pytest.fixture(scope="module")
def file_adapter_type() -> str:
    return "json"


def test_default_import(import_mode, from_file):
    pass


def test_import_with_connected_user(import_mode, from_file, connected_user):
    pass


def test_import_with_disconnected_user(import_mode, from_file, disconnected_user):
    pass


def test_export_file_adapter(import_mode, from_file, file_adapter_type):
    pass



from pattern_singleton import Singleton

from chpass.dal.db_connection import DBConnection
from chpass.dal.table_adapters.downloads_table_adapter import DownloadsTableAdapter
from chpass.dal.table_adapters.history_table_adapter import HistoryTableAdapter


class HistoryDBAdapter(metaclass=Singleton):
    def __init__(self, db_connection: DBConnection) -> None:
        self._db_connection = db_connection
        self._history_table_adapter = HistoryTableAdapter(self._db_connection)
        self._downloads_table_adapter = DownloadsTableAdapter(self._db_connection)

    @property
    def history_table(self) -> HistoryTableAdapter:
        return self._history_table_adapter

    @property
    def downloads_table(self) -> DownloadsTableAdapter:
        return self._downloads_table_adapter

    def close(self) -> None:
        self._db_connection.close()





class FileAdapterNotSupportedException(Exception):
    def __init__(self, file_adapter_format: str) -> None:
        self._file_adapter_format = file_adapter_format
        message = f"the following file adapter format is not supported - {file_adapter_format}"
        super().__init__(message)

    @property
    def file_adapter_format(self) -> str:
        return self._file_adapter_format

    @file_adapter_format.setter
    def file_adapter_format(self, new_file_adapter_format: str) -> None:
        self._file_adapter_format = new_file_adapter_format



from pattern_singleton import Singleton

from chpass.dal.db_adapters.history_db_adapter import HistoryDBAdapter
from chpass.dal.db_adapters.logins_db_adapter import LoginsDBAdapter
from chpass.dal.db_adapters.top_sites_db_adapter import TopSitesDBAdapter


class ChromeDBAdapter(metaclass=Singleton):
    def __init__(
            self,
            logins_db_adapter: LoginsDBAdapter,
            history_db_adapter: HistoryDBAdapter,
            top_sites_db_adaper: TopSitesDBAdapter
    ):
        self._logins_db_adapter = logins_db_adapter
        self._history_db_adapter = history_db_adapter
        self._top_sites_db_adaper = top_sites_db_adaper

    @property
    def logins_db(self) -> LoginsDBAdapter:
        return self._logins_db_adapter

    @property
    def history_db(self) -> HistoryDBAdapter:
        return self._history_db_adapter

    @property
    def top_sites_db(self) -> TopSitesDBAdapter:
        return self._top_sites_db_adaper

    def close(self) -> None:
        for db_adapter in [self._logins_db_adapter, self._history_db_adapter, self._top_sites_db_adaper]:
            db_adapter.close()



from pattern_singleton import Singleton

from chpass.dal.db_connection import DBConnection
from chpass.dal.table_adapters.top_sites_table_adapter import TopSitesTableAdapter


class TopSitesDBAdapter(metaclass=Singleton):
    def __init__(self, db_connection: DBConnection) -> None:
        self._db_connection = db_connection
        self._top_sites_table_adapter = TopSitesTableAdapter(self._db_connection)

    @property
    def top_sites_table(self) -> TopSitesTableAdapter:
        return self._top_sites_table_adapter

    def close(self) -> None:
        self._db_connection.close()



from dataclasses import dataclass

from sqlalchemy import Integer, String, Column

from chpass.dal.models.base import Base


@dataclass
class TopSite(Base):
    __tablename__ = 'top_sites'
    url = Column(String, primary_key=True)
    url_rank = Column(Integer)
    title = Column(String)
    redirects = Column(String)



import pytest
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import NoResultFound

from chpass.dal.models.login import Login
from chpass.exceptions.login_not_found_exception import LoginNotFoundException


@pytest.fixture(scope="module")
def invalid_login() -> int:
    return -1


@pytest.fixture(scope="module")
def login() -> dict:
    return {
        "origin_url": "test",
        "action_url": "test",
        "username_element": "test",
        "username_value": "test",
        "password_element": "test",
        "password_value": bytes(),
        "submit_element": "test",
        "signon_realm": "test",
        "date_created": 0,
        "blacklisted_by_user": 0,
        "scheme": 0,
        "password_type": 0,
        "times_used": 0,
        "form_data": bytes(),
        "date_synced": 0,
        "display_name": "test",
        "icon_url": "test",
        "federation_url": "test",
        "skip_zero_click": 0,
        "generation_upload_status": 0,
        "possible_username_pairs": bytes(),
        "date_last_used": 0,

    }


@pytest.fixture(scope="module")
def login_id_not_exist() -> int:
    return -1


@pytest.fixture(scope="module")
def invalid_login_id() -> str:
    return "invalid"


def test_get_logins(chrome_db_adapter):
    logins = chrome_db_adapter.logins_db.logins_table.get_all_logins()
    assert all(isinstance(login, Login) for login in logins)


def test_get_logins_serializable(chrome_db_adapter):
    logins = chrome_db_adapter.logins_db.logins_table.get_all_logins(serializable=True)
    assert all(isinstance(login, dict) for login in logins)


def test_insert_login(chrome_db_adapter, login):
    db_logins = chrome_db_adapter.logins_db.logins_table.get_all_logins()
    db_login_ids = [db_login.id for db_login in db_logins]
    login["id"] = max(db_login_ids) + 1
    chrome_db_adapter.logins_db.logins_table.insert_login(login)
    updated_db_logins = chrome_db_adapter.logins_db.logins_table.get_all_logins(serializable=True)
    assert login in updated_db_logins


def test_insert_invalid_login(chrome_db_adapter, invalid_login):
    with pytest.raises(TypeError):
        chrome_db_adapter.logins_db.logins_table.insert_login(invalid_login)


def test_insert_exist_login(chrome_db_adapter, login):
    with pytest.raises(IntegrityError):
        chrome_db_adapter.logins_db.logins_table.insert_login(login)


def test_get_login(chrome_db_adapter):
    login = chrome_db_adapter.logins_db.logins_table.get_all_logins()[0]
    result = chrome_db_adapter.logins_db.logins_table.get_login(login.id)
    assert result == login


def test_get_invalid_login(chrome_db_adapter, invalid_login_id):
    with pytest.raises(LoginNotFoundException):
        chrome_db_adapter.logins_db.logins_table.get_login(invalid_login_id, serializable=True)


def test_get_login_not_exist(chrome_db_adapter, login_id_not_exist):
    with pytest.raises(LoginNotFoundException):
        chrome_db_adapter.logins_db.logins_table.get_login(login_id_not_exist, serializable=True)


def test_delete_invalid_login_id(chrome_db_adapter, invalid_login_id):
    with pytest.raises(NoResultFound):
        chrome_db_adapter.logins_db.logins_table.delete_login(invalid_login_id)


def test_delete_login_not_exist(chrome_db_adapter, login_id_not_exist):
    with pytest.raises(NoResultFound):
        chrome_db_adapter.logins_db.logins_table.delete_login(login_id_not_exist)


def test_delete_login(chrome_db_adapter):
    login_id = chrome_db_adapter.logins_db.logins_table.get_all_logins()[-1].id
    chrome_db_adapter.logins_db.logins_table.delete_login(login_id)
    test_delete_login_not_exist(chrome_db_adapter, login_id)





class LoginNotFoundException(Exception):
    def __init__(self, login_id: int) -> None:
        self._login_id = login_id
        message = f"login with id {self._login_id} not found in the db"
        super().__init__(message)

    @property
    def login_id(self) -> int:
        return self._login_id

    @login_id.setter
    def login_id(self, new_login_id: int) -> None:
        self._login_id = new_login_id



import pytest


@pytest.fixture(scope="module")
def export_mode() -> str:
    return "export"


@pytest.fixture(scope="module")
def file_adapter_type() -> str:
    return "json"


@pytest.fixture(scope="module")
def destination_folder() -> str:
    return "dist"


def test_default_export(export_mode):
    pass


def test_export_with_connected_user(export_mode, connected_user):
    pass


def test_export_with_disconnected_user(export_mode, disconnected_user):
    pass


def test_export_file_adapter(export_mode, file_adapter_type):
    pass


def test_export_destination_folder(export_mode, destination_folder):
    pass





class OperatingSystemNotSupported(Exception):
    def __init__(self, operating_system: str) -> None:
        self._operating_system = operating_system
        message = f"operating system not supported - {operating_system}"
        super().__init__(message)

    @property
    def operating_system(self) -> str:
        return self._operating_system

    @operating_system.setter
    def operating_system(self, new_operating_system: str) -> None:
        self._operating_system = new_operating_system



import pytest
from sqlalchemy.exc import ArgumentError, NoSuchModuleError

from chpass.dal.db_connection import DBConnection
from chpass.services.path import get_chrome_logins_path


@pytest.fixture(scope="module")
def db_connection() -> DBConnection:
    return DBConnection()


@pytest.fixture(scope="module")
def db_protocol() -> str:
    return "sqlite"


@pytest.fixture(scope="module")
def db_protocol_not_exist() -> str:
    return "not_exist"


@pytest.fixture(scope="module")
def db_file_path(connected_user) -> str:
    return get_chrome_logins_path(connected_user)


def test_db_connection_close_fail(db_connection):
    with pytest.raises(AttributeError):
        db_connection.close()


def test_db_connection_connect(db_connection, db_protocol, db_file_path):
    db_connection.connect(db_protocol, db_file_path)


@pytest.mark.parametrize("db_protocol", [-1])
@pytest.mark.parametrize("db_file_path", [-1, "not_exist"])
def test_db_connection_connect_invalid_protocol(db_connection, db_protocol, db_file_path):
    with pytest.raises(ArgumentError):
        db_connection.connect(db_protocol, db_file_path)


def test_db_connection_connect_protocol_not_exist(db_connection, db_protocol_not_exist, db_file_path):
    with pytest.raises(NoSuchModuleError):
        db_connection.connect(db_protocol_not_exist, db_file_path)


def test_db_connection_close(db_connection):
    db_connection.close()



from typing import List


class IFileAdapter(object):
    def write(self, data: List[dict], output_file_path: str, byte_columns: list = None) -> None:
        raise NotImplementedError

    def read(self, path: str, byte_columns: list = None) -> list:
        raise NotImplementedError



import os

CHROME_FOLDER_OS_PATHS = {
    "win32": r"AppData\Local\Google\Chrome\User Data",
    "linux": ".config/google-chrome",
    "linux2": ".config/google-chrome",
    "darwin": "Library/Application Support/Google/Chrome"
}

LOGINS_DB_FILE_PATH = os.path.join("Default", "Login Data")
HISTORY_DB_FILE_PATH = os.path.join("Default", "History")
TOP_SITES_DB_FILE_PATH = os.path.join("Default", "Top Sites")
GOOGLE_PICTURE_FILE_PATH = os.path.join("Default", "Google Profile Picture.png")

DEFAULT_EXPORT_DESTINATION_FOLDER = "dist"
OUTPUT_FILE_PATHS = {
    "csv": {
        "passwords": "passwords.csv",
        "history": "history.csv",
        "downloads": "downloads.csv",
        "top_sites": "top_sites.csv",
        "profile_picture": "profile.jpg"
    },
    "json": {
        "passwords": "passwords.json",
        "history": "history.json",
        "downloads": "downloads.json",
        "top_sites": "top_sites.json",
        "profile_picture": "profile.jpg"
    }
}
DB_PROTOCOL = "sqlite"

DEFAULT_FILE_ADAPTER = "csv"
CSV_ADAPTER_ENCODING = "utf-8"
DEFAULT_USER = "~"
WRITE_FILE_MODE = "w"

PASSWORDS_FILE_BYTES_COLUMNS = ["form_data", "password_value", "possible_username_pairs"]

CREDENTIALS_ALREADY_EXIST_MESSAGE = "Warning: credentials already exist on chrome for this user. url: {}"



from dataclasses import dataclass

from sqlalchemy import Integer, String, Column, LargeBinary

from chpass.dal.models.base import Base


@dataclass
class Login(Base):
    __tablename__ = 'logins'
    origin_url = Column(String)
    action_url = Column(String)
    username_element = Column(String)
    username_value = Column(String)
    password_element = Column(String)
    password_value = Column(LargeBinary)
    submit_element = Column(String)
    signon_realm = Column(String)
    date_created = Column(Integer)
    blacklisted_by_user = Column(Integer)
    scheme = Column(Integer)
    password_type = Column(Integer)
    times_used = Column(Integer)
    form_data = Column(LargeBinary)
    date_synced = Column(Integer)
    display_name = Column(String)
    icon_url = Column(String)
    federation_url = Column(String)
    skip_zero_click = Column(Integer)
    generation_upload_status = Column(Integer)
    possible_username_pairs = Column(LargeBinary)
    id = Column(Integer, primary_key=True)
    date_last_used = Column(Integer)



from chpass.dal.models.topSite import TopSite


def test_get_top_sites(chrome_db_adapter):
    top_sites = chrome_db_adapter.top_sites_db.top_sites_table.get_top_sites()
    assert all(isinstance(top_site, TopSite) for top_site in top_sites)


def test_get_top_sites_serializable(chrome_db_adapter):
    top_sites = chrome_db_adapter.top_sites_db.top_sites_table.get_top_sites(serializable=True)
    assert all(isinstance(top_site, dict) for top_site in top_sites)



# -*- coding: utf-8 -*-
"""
    chpass
    ~~~~~
    chpass is a package for gather information from Chrome.
    :copyright: 2020 Ben Gabay
    :license: MIT, see LICENSE for more details.
"""
from .__main__ import start

from .services.chrome import (
    import_chrome_passwords,
    export_chrome_data,
    export_profile_picture,
    export_downloads,
    export_top_sites,
    export_history,
    export_passwords
)
from .services.path import get_chrome_user_folder, get_home_directory


def main():
    start()



from chpass.dal.db_connection import DBConnection
from chpass.dal.models.login import Login
from chpass.exceptions.login_not_found_exception import LoginNotFoundException


class LoginsTableAdapter(object):
    def __init__(self, db_connection: DBConnection) -> None:
        self._db_connection = db_connection

    def get_all_logins(self, serializable: bool = False) -> list:
        return self._db_connection.select(Login, serializable=serializable)

    def get_login(self, login_id: int, serializable: bool = False) -> Login:
        filters = (Login.id == login_id,)
        result = self._db_connection.select(Login, filters, serializable=serializable)
        if result:
            return result[0]
        else:
            raise LoginNotFoundException(login_id)

    def insert_login(self, credentials: dict) -> None:
        login = Login(**credentials)
        self._db_connection.insert(login)

    def delete_login(self, login_id: int) -> None:
        filters = (Login.id == login_id,)
        self._db_connection.delete(Login, filters)



import getpass
import os
from shutil import copyfile
from typing import List, Callable

from chpass.dal.chrome_db_adapter import ChromeDBAdapter
from chpass.core.interfaces import file_adapter_interface
from chpass.config import PASSWORDS_FILE_BYTES_COLUMNS, CREDENTIALS_ALREADY_EXIST_MESSAGE
from chpass.services.path import get_chrome_profile_picture_path


def generic_export(
        get_data_func: Callable,
        file_adapter: file_adapter_interface,
        destination_folder: str,
        filename: str) -> None:
    data = get_data_func(serializable=True)
    file_adapter.write(data, f"{destination_folder}/{filename}")


def export_profile_picture(destination_path: str, user: str = getpass.getuser()) -> None:
    """Exports google profile picture
    :param destination_path: Destination path to export the picture
    :param user: Chrome user
    :return: None
    :rtype: None
    """
    source_path = get_chrome_profile_picture_path(user)
    copyfile(source_path, destination_path)


def export_history(
        chrome_db_adapter: ChromeDBAdapter,
        file_adapter: file_adapter_interface,
        destination_folder: str,
        filename: str) -> None:
    """Exports chrome history to a file
    :param chrome_db_adapter: Adapter for the chrome db
    :param file_adapter: Adapter for writing the history data to a file
    :param destination_folder: Destination folder path to save file in
    :param filename: Destination file name for the history
    :return: None
    :rtype: None
    """
    generic_export(chrome_db_adapter.history_db.history_table.get_chrome_history, file_adapter, destination_folder,
                   filename)


def export_downloads(
        chrome_db_adapter: ChromeDBAdapter,
        file_adapter: file_adapter_interface,
        destination_folder: str,
        filename: str) -> None:
    """Exports chrome downloads to a file
    :param chrome_db_adapter: Adapter for the chrome db
    :param file_adapter: Adapter for writing the downloads data to a file
    :param destination_folder: Destination folder path to save file in
    :param filename: Destination file name for the downloads
    :return: None
    :rtype: None
    """
    generic_export(chrome_db_adapter.history_db.downloads_table.get_chrome_downloads, file_adapter, destination_folder,
                   filename)


def export_top_sites(
        chrome_db_adapter: ChromeDBAdapter,
        file_adapter: file_adapter_interface,
        destination_folder: str,
        filename: str) -> None:
    """Exports chrome top sites to a file
    :param chrome_db_adapter: Adapter for the chrome db
    :param file_adapter: Adapter for writing the top sites data to a file
    :param destination_folder: Destination folder path to save file in
    :param filename: Destination file name for the top sites
    :return: None
    :rtype: None
    """
    generic_export(chrome_db_adapter.top_sites_db.top_sites_table.get_top_sites, file_adapter, destination_folder,
                   filename)


def export_passwords(
        chrome_db_adapter: ChromeDBAdapter,
        file_adapter: file_adapter_interface,
        destination_folder: str,
        filename: str) -> None:
    """Exports chrome passwords to a file
    :param chrome_db_adapter: Adapter for the chrome db
    :param file_adapter: Adapter for writing the passwords data to a file
    :param destination_folder: Destination folder path to save file in
    :param filename: Destination file name for the passwords
    :return: None
    :rtype: None
    """
    logins = chrome_db_adapter.logins_db.logins_table.get_all_logins(serializable=True)
    file_adapter.write(logins, f"{destination_folder}/{filename}", byte_columns=PASSWORDS_FILE_BYTES_COLUMNS)


def export_chrome_data(
        chrome_db_adapter: ChromeDBAdapter,
        destination_folder: str,
        file_adapter: file_adapter_interface,
        output_file_paths: dict,
        user: str = getpass.getuser(),
        export_kind: str = None) -> None:
    """Exports chrome data to a file
    :param chrome_db_adapter: Adapter for the chrome db
    :param destination_folder: Destination folder path to save file in
    :param file_adapter: Adapter for writing the data to a file
    :param output_file_paths: Dictionary that maps between data type and its destination file path
    :param user: Chrome user
    :param export_kind: Specific data type export instead of export all the data
    :return: None
    :rtype: None
    """
    if not os.path.exists(destination_folder):
        os.mkdir(destination_folder)
    export_functions = {
        "passwords": lambda: export_passwords(chrome_db_adapter, file_adapter, destination_folder,
                                              output_file_paths["passwords"]),
        "history": lambda: export_history(chrome_db_adapter, file_adapter, destination_folder,
                                          output_file_paths["history"]),
        "downloads": lambda: export_downloads(chrome_db_adapter, file_adapter, destination_folder,
                                              output_file_paths["downloads"]),
        "top_sites": lambda: export_top_sites(chrome_db_adapter, file_adapter, destination_folder,
                                              output_file_paths["top_sites"]),
        "profile_pic": lambda: export_profile_picture(f"{destination_folder}/{output_file_paths['profile_picture']}", user)
    }
    if export_kind:
        export_functions[export_kind]()
    else:
        for export_func in export_functions.values():
            export_func()


def filter_existed_logins(chrome_db_adapter: ChromeDBAdapter, logins_to_import: List[dict]) -> list:
    db_logins = chrome_db_adapter.logins_db.logins_table.get_all_logins(serializable=True)
    db_logins_signon_realms = [db_login["signon_realm"] for db_login in db_logins]
    unique_logins = []
    for login in logins_to_import:
        if login["signon_realm"] not in db_logins_signon_realms:
            unique_logins.append(login)
        else:
            print(CREDENTIALS_ALREADY_EXIST_MESSAGE.format(login["signon_realm"]))
    return unique_logins


def import_chrome_passwords(chrome_db_adapter: ChromeDBAdapter, source_file_path: str, file_adapter: file_adapter_interface) -> None:
    """Imports passwords to chrome db.
    :param chrome_db_adapter: Adapter for the chrome db
    :param source_file_path: Source file to import the passwords from
    :param file_adapter: Adapter to read the passwords from a file
    :return: None
    :rtype: None
    """
    if not os.path.exists(source_file_path):
        raise FileNotFoundError(source_file_path)
    logins_to_import = file_adapter.read(source_file_path, byte_columns=PASSWORDS_FILE_BYTES_COLUMNS)
    unique_logins_to_import = filter_existed_logins(chrome_db_adapter, logins_to_import)
    for login in unique_logins_to_import:
        chrome_db_adapter.logins_db.logins_table.insert_login(login)



from chpass.dal.models.download import Download


def test_get_downloads(chrome_db_adapter):
    downloads = chrome_db_adapter.history_db.downloads_table.get_chrome_downloads()
    assert all(isinstance(download, Download) for download in downloads)


def test_get_downloads_serializable(chrome_db_adapter):
    downloads = chrome_db_adapter.history_db.downloads_table.get_chrome_downloads(serializable=True)
    assert all(isinstance(download, dict) for download in downloads)



from typing import List

import pandas as pd

from chpass.core.interfaces.file_adapter_interface import IFileAdapter
from chpass.services.bytes import convert_bytes_in_data


class JsonFileAdapter(IFileAdapter):
    def write(self, data: List[dict], output_file_path: str, byte_columns: list = None) -> None:
        convert_bytes_in_data(data, list, byte_columns)
        df = pd.DataFrame(data)
        df.to_json(output_file_path)

    def read(self, path: str, byte_columns: list = None) -> list:
        df = pd.read_json(path)
        data = [dict(row[1]) for row in df.iterrows()]
        convert_bytes_in_data(data, bytes, byte_columns)
        return data



import os

import pytest

from chpass.exceptions.operating_system_not_supported import OperatingSystemNotSupported
from chpass.exceptions.user_not_found_exception import UserNotFoundException
from chpass.services.path import get_home_directory, get_chrome_user_folder


@pytest.fixture(scope="module")
def invalid_os() -> int:
    return -1


@pytest.fixture(scope="module")
def os_not_exist() -> str:
    return "ChromeOS"


def test_get_home_directory(connected_user):
    home_directory = get_home_directory(connected_user)
    assert os.path.basename(home_directory) == connected_user
    assert os.path.exists(home_directory)


@pytest.fixture
def user_not_exist() -> str:
    return "not_exist"


def test_get_home_directory_user_not_exist(user_not_exist):
    with pytest.raises(UserNotFoundException):
        get_home_directory(user_not_exist)


@pytest.fixture
def invalid_user() -> int:
    return -1


def test_get_home_directory_invalid_user(invalid_user):
    with pytest.raises(TypeError):
        get_home_directory(invalid_user)


def test_get_chrome_user_folder(connected_user):
    chrome_user_folder = get_chrome_user_folder(connected_user)
    assert os.path.exists(chrome_user_folder)


def test_get_chrome_user_folder_os_not_exist(connected_user, os_not_exist):
    with pytest.raises(OperatingSystemNotSupported):
        get_chrome_user_folder(connected_user, platform=os_not_exist)


def test_get_chrome_user_folder_invalid_os(connected_user, invalid_os):
    with pytest.raises(OperatingSystemNotSupported):
        get_chrome_user_folder(connected_user, platform=invalid_os)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="chpass",
    version="0.2.2",
    author="Ben Gabay",
    author_email="ben.gabay38@gmail.com",
    license="License :: OSI Approved :: MIT License",
    description="Gather information from chrome",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bengabay11/chpass",
    packages=setuptools.find_packages(),
    install_requires=[
        "sqlalchemy==1.3.18",
        "pandas==1.3.1",
        "pattern_singleton==1.2.0"
    ],
    python_requires=">=3",
    entry_points={
        'console_scripts': ['chpass = chpass.__init__:main'],
    },
    keywords="chrome passwords",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



import getpass

import pytest

from chpass.__main__ import create_chrome_db_adapter
from chpass.config import DB_PROTOCOL
from chpass.dal import chrome_db_adapter


@pytest.fixture(scope="session")
def connected_user() -> str:
    return getpass.getuser()


@pytest.fixture(scope="session")
def disconnected_user() -> str:
    return "Administrator"


@pytest.fixture(scope="session")
def chrome_db_adapter(request, connected_user) -> chrome_db_adapter:
    chrome_db_adapter = create_chrome_db_adapter(DB_PROTOCOL, connected_user)

    def fin():
        chrome_db_adapter.close()

    request.addfinalizer(fin)
    return chrome_db_adapter



import argparse
import getpass

from chpass.config import (
    DEFAULT_EXPORT_DESTINATION_FOLDER,
    DEFAULT_FILE_ADAPTER
)


def create_import_parser(subparsers: argparse._SubParsersAction) -> None:
    parser_import = subparsers.add_parser("import", description="imports a file with the passwords")
    parser_import.add_argument(
        "-f",
        "--from",
        dest="from_file",
        help="credentials file to import from",
        type=str,
        required=True
    )


def create_export_parser(subparsers: argparse._SubParsersAction) -> None:
    parser_export = subparsers.add_parser("export", description="exports a chrome data files")
    parser_export.add_argument(
        "-d",
        "--destination",
        dest="destination_folder",
        type=str,
        help="destination folder to export the files",
        default=DEFAULT_EXPORT_DESTINATION_FOLDER
    )
    export_subparsers = parser_export.add_subparsers(dest="export_kind")
    export_subparsers.required = False
    export_subparsers.add_parser("passwords", description="export passwords")
    export_subparsers.add_parser("history", description="export history")
    export_subparsers.add_parser("downloads", description="export downloads")
    export_subparsers.add_parser("top_sites", description="export top sites")
    export_subparsers.add_parser("profile_pic", description="export profile picture")


def create_arg_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="chpass",
        description="Gather information from chrome",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    subparsers = parser.add_subparsers(dest="mode")
    subparsers.required = True
    create_import_parser(subparsers)
    create_export_parser(subparsers)
    parser.add_argument("-u", "--user", dest="user", type=str, default=getpass.getuser())
    parser.add_argument("-i", "--file-adapter", dest="file_adapter", type=str, default=DEFAULT_FILE_ADAPTER)
    return parser


def parse_args(args: list) -> argparse.Namespace:
    arg_parser = create_arg_parser()
    return arg_parser.parse_args(args)



from dataclasses import dataclass

from sqlalchemy.ext.declarative import as_declarative, declared_attr


@dataclass
@as_declarative()
class Base(object):
    @declared_attr
    def __tablename__(cls) -> str:
        return cls.__name__.lower()

    def json(self) -> dict:
        dict_base = self.__dict__.copy()
        for key in dict_base.copy():
            if key.startswith("_"):
                del dict_base[key]
        return dict_base



from typing import Type, Any

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from chpass.dal.models.base import Base
from chpass.dal.session import session_scope


class DBConnection(object):
    def __init__(self) -> None:
        self._connection = None
        self._session_class = None

    def connect(self, protocol: str, database: str) -> None:
        url = f"{protocol}://{database}"
        engine = create_engine(url)
        self._session_class = sessionmaker(bind=engine)
        self._connection = engine.connect()

    def select(self, model: Type[Base], filters: tuple = (), serializable: bool = False) -> list:
        with session_scope(self._session_class) as session:
            query = session.query(model).filter(*filters)
            results = query.all()
            return [result.json() for result in results] if serializable else results

    def insert(self, row: Base) -> None:
        with session_scope(self._session_class) as session:
            session.add(row)

    def update(self, model: Type[Base], filters: tuple, column: Any, new_value: Any) -> None:
        with session_scope(self._session_class) as session:
            row = session.query(model).filter(*filters).one()
            setattr(row, column, new_value)

    def delete(self, model: Type[Base], filters: tuple) -> None:
        with session_scope(self._session_class) as session:
            row = session.query(model).filter(*filters).one()
            session.delete(row)

    def close(self) -> None:
        self._connection.close()



import os

import pytest

from chpass.exceptions.user_not_found_exception import UserNotFoundException
from chpass import export_profile_picture


@pytest.fixture(scope="module")
def correct_destination_path() -> str:
    return "test_profile_picture.jpg"


def test_export_profile_picture(correct_destination_path, connected_user):
    export_profile_picture(correct_destination_path, connected_user)
    assert os.path.exists(correct_destination_path)
    os.remove(correct_destination_path)


@pytest.fixture
def user_not_exist() -> str:
    return "not_exist"


def test_export_profile_picture_user_not_exist(correct_destination_path, user_not_exist):
    with pytest.raises(UserNotFoundException):
        export_profile_picture(user_not_exist, correct_destination_path)
    assert not os.path.exists(correct_destination_path)


@pytest.fixture
def invalid_user() -> int:
    return -1


def test_export_profile_picture_invalid_user(correct_destination_path, invalid_user):
    with pytest.raises(TypeError):
        export_profile_picture(correct_destination_path, invalid_user)
    assert not os.path.exists(correct_destination_path)


@pytest.fixture
def destination_path_not_exist() -> str:
    return "not_exist/test_profile_picture.jpg"


def test_export_profile_picture_destination_path_not_exist(destination_path_not_exist, connected_user):
    with pytest.raises(FileNotFoundError):
        export_profile_picture(destination_path_not_exist, connected_user)
    assert not os.path.exists(destination_path_not_exist)


@pytest.fixture
def invalid_destination_path() -> int:
    return -1


def test_export_profile_picture_invalid_destination_path(invalid_destination_path, connected_user):
    with pytest.raises(ValueError):
        export_profile_picture( invalid_destination_path, connected_user)
    assert not os.path.exists(invalid_destination_path)
